#Compute Blaker's exact confidence interval for the Binomial distribution
#Auteurs/Authors - Bruno LECOUTRE & Jacques POITEVINEAU
#Reference: 
#  Lecoutre B., Poitevineau J. (2012)
#  New results for computing exact confidence intervals for one parameter discrete distributions.
#  hal.archives ouvertes.
#  http://hal.archives-ouvertes.fr/docs/00/91/14/88/PDF/Bruno_Lecoutre_Jacques_Poitevineau_-_New_results_for_computing_exact_confidence_intervals_for_one_parameter_discrete_distributions.pdf 

bbinomlim <- function(epsilon,alpha,x,n){ 
    l <- 1-bbinomsup(epsilon,alpha,n-x,n)
    u <- bbinomsup(epsilon,alpha,x,n)
    print (c(x,n,l,u))
}

bbinomlist <- function(epsilon,alpha,x,n){ 
  for(i in x:n) { 
    l <- 1-bbinomsup(epsilon,alpha,i-x,i)
    u <- bbinomsup(epsilon,alpha,x,i)
    #u <- bbinomsupmon(epsilon,alpha,x,i)
    print (c(i,l,u))
  }	
}

bbinomsupmon <- function(epsilon,alpha,x,n){
  if (x==n) return(1)
  #Search for non-monotonocity in n
  u <- bbinomsup(epsilon,alpha,x,n)
  k <- 1
  while (k<100)
    {u1 <- bbinomsup(epsilon,alpha,x,n+k)
     if (u1 > u) u <- u1   #correction for non-monotonocity
     p <- pbinom(x,n+k,u)
     if (p < alpha/2) return(u)  #p cannot be larger than alpha/2
     k <- k + 1}}

bbinomsupcoh <- function(epsilon,alpha,x,n){
  if (x==n) return(1)
  #U_alpha/2
  u2 <- qbeta(1-alpha/2,x+1,n-x)
  #Search for non-monotonocity in n
  u <- bbinomsup(epsilon,alpha,x,n)
  k <- 1
  while (k<n)
    {u1 <- bbinomsup(epsilon,alpha,x,n-k)
     if (u1 < u) u <- u1    #correction for non-monotonocity
     if (u1>u2) return(u)  #u1 cannot be larger than U_alpha/2
     k <- k + 1}}

bbinomsup <- function(epsilon,alpha,x,n)
{
  if (x==n) return(1)
  #U_alpha and U_alpha/2
  u1 <- qbeta(1-alpha,x+1,n-x)
    #particular case x = n-1
    if (x==n-1) return(u1)
  u2 <- qbeta(1-alpha/2,x+1,n-x)
  #x^+
  xp <- qbinom(1-alpha/2,n,u2)+1
  #particular case x^+ = n+1
  if (xp==n+1){
     p <- dbinom(n,n,u1)
     if (p>=alpha) return(u1)}
  #\check\theta
  if (xp<=n){
      lw <- lgamma(xp) + lgamma(n-xp+1) - lgamma(x+1) - lgamma(n-x)
      w <- exp(lw/(xp-x-1))
      thetamin <- w/(1+w)}
	else {thetamin <- u1}
  u <- upperlimit(alpha,epsilon,u1,u2,thetamin,x,xp,n)
  return(u)}

upperlimit <- function(alpha,epsilon,u1,u2,thetamin,x,xp,arg){
  #x^- = x^+ ?
  pq <- sumdif(u1,arg,x,xp-1)

  if (pq[2]>0){
  #solve Delta = 0
     umin=u1
     umax=u2
     pq[2] <- 1
     while (abs(pq[2])>epsilon){
        u= (umin + umax)/2
        pq <- sumdif(u,arg,x,xp-1)
        if (pq[2]>0){umin <- u} else {umax <- u}}
     pq <- sumdif(u,arg,x,xp)
     if (pq[1]<alpha) return(u)}

#solve ksi = alpha
  umin=u1
  umax=min(u2,thetamin)
  pq[1] <- 2
  while (abs(pq[1]-alpha)>epsilon){
  u= (umin + umax)/2
  pq <- sumdif(u,arg,x,xp)
  if (pq[1]>alpha){umin <- u} else {umax <- u}}
  return(u)}

sumdif <- function(theta,arg,x,y){
   p <- pbinom(x,arg,theta)
   q <- 1-pbinom(y-1,arg,theta)
   sumdif <- c(p+q,p-q)}
