#Compute Blaker's exact confidence interval for the Negative Binomial distribution
#Auteurs/Authors - Bruno LECOUTRE & Jacques POITEVINEAU
#Reference: 
#  Lecoutre B., Poitevineau J. (2012)
#  New results for computing exact confidence intervals for one parameter discrete distributions.
#  hal.archives ouvertes.
#  http://hal.archives-ouvertes.fr/docs/00/91/14/88/PDF/Bruno_Lecoutre_Jacques_Poitevineau_-_New_results_for_computing_exact_confidence_intervals_for_one_parameter_discrete_distributions.pdf 

bnegbinomlim <- function(epsilon,alpha,x,s){ 
    l <- bnegbinominf(epsilon,alpha,x,s)
    u <- bnegbinomsup(epsilon,alpha,x,s)
    print (c(x,s,l,u))
}

bnegbinomlist <- function(epsilon,alpha,x,s){ 
  for(i in 1:s) { 
    l <- bnegbinominf(epsilon,alpha,x,i)
    u <- bnegbinomsup(epsilon,alpha,x,i)
    print (c(i,l,u))
  }	
}

bnegbinomsup <- function(epsilon,alpha,x,s){
  #particular case x = 0
  if (x==0) return(1)
  #U_alpha and U_alpha/2
  u1 <- qbeta(1-alpha,s,x)
     #particular case x = 1
     if (x==1) return(u1)
  u2 <- qbeta(1-alpha/2,s,x)
  #x^+
  xp <- -qnbinom(alpha/2,s,u2)+1
  #particular case x^+ = 1
  if (xp==1){
       p <- dnbinom(0,s,u1)
       if (p>=alpha) return(u1)}
  #\check\theta
  if (xp < 1){thetamin <- 1 - exp((lgamma(-xp+1)+lgamma(s+x)-lgamma(x)-lgamma(s-xp+1))/(-xp-x+1))}
        else {thetamin <- u1}
  u <- upperlimit(1,alpha,epsilon,u1,u2,thetamin,-x,xp,s)
  return(u)}

bnegbinominf <- function(epsilon,alpha,x,s){
  #L_alpha and L_alpha/2
  l1 <- qbeta(1-alpha,x+1,s)
  l2 <- qbeta(1-alpha/2,x+1,s)
  #x^+
  xp <- qnbinom(1-alpha/2,s,1-l2)+1
  #\check\theta
  thetamin <- exp((lgamma((xp))+lgamma(s+x+1)-lgamma(x+1)-lgamma(s+xp))/(xp-x-1))
  u <- upperlimit(0,alpha,epsilon,l1,l2,thetamin,x,xp,s)
  return(1-u)}

upperlimit <- function(uplow,alpha,epsilon,u1,u2,thetamin,x,xp,arg){
  #x^- = x^+ ?
  pq <- sumdif(uplow,u1,arg,x,xp-1)

  if (pq[2]>0){
  #solve Delta = 0
     umin=u1
     umax=u2
     pq[2] <- 1
     while (abs(pq[2])>epsilon){
        u= (umin + umax)/2
        pq <- sumdif(uplow,u,arg,x,xp-1)
        if (pq[2]>0){umin <- u} else {umax <- u}}
     pq <- sumdif(uplow,u,arg,x,xp)
     if (pq[1]<alpha) return(u)}

#solve ksi = alpha
  umin=u1
  umax=min(u2,thetamin)
  pq[1] <- 2
  while (abs(pq[1]-alpha)>epsilon){
  u= (umin + umax)/2
  pq <- sumdif(uplow,u,arg,x,xp)
  if (pq[1]>alpha){umin <- u} else {umax <- u}}
  return(u)}

sumdif <- function(uplow,theta,s,x,y){
  if (uplow == 1) #upper
	{p <- 1-pnbinom(-x-1,s,theta)
       q <- pnbinom(-y,s,theta)}
  else #lower
	{p <- pnbinom(x,s,1-theta)
       q <- 1-pnbinom(y-1,s,1-theta)}
  sumdif <- c(p+q,p-q)}