#Compute Blaker's exact confidence interval for the Poisson distribution
#Auteurs/Authors - Bruno LECOUTRE & Jacques POITEVINEAU
#Reference: 
#  Lecoutre B., Poitevineau J. (2012)
#  New results for computing exact confidence intervals for one parameter discrete distributions.
#  hal.archives ouvertes.
#  http://hal.archives-ouvertes.fr/docs/00/91/14/88/PDF/Bruno_Lecoutre_Jacques_Poitevineau_-_New_results_for_computing_exact_confidence_intervals_for_one_parameter_discrete_distributions.pdf 

bpoissonlim <- function(epsilon,alpha,x,n){ 
    l <- bpoissoninf(epsilon,alpha,x,n)
    u <- bpoissonsup(epsilon,alpha,x,n)
    print (c(x,n,l,u))
}

bpoissonlist <- function(epsilon,alpha,x,n){ 
  for(i in 1:n) { 
    l <- bpoissoninf(epsilon,alpha,x,i)
    u <- bpoissonsup(epsilon,alpha,x,i)
    print (c(i,l,u))
  }
}

bpoissonsup <- function(epsilon,alpha,x,n){
  #U_alpha and U_alpha/2
   u1 <- qgamma(1-alpha,x+1)/n
   u2 <- qgamma(1-alpha/2,x+1)/n
  #x^+
   xp <- qpois(1-alpha/2,n*u2)+1
  #\check\theta
   thetamin <- exp((lgamma(xp)-lgamma(x+1))/(xp-x-1)-log(n))
   u <- upperlimit(1,alpha,epsilon,u1,u2,thetamin,x,xp,n)
   return(u)}

bpoissoninf <- function(epsilon,alpha,x,n){
  #particular case x = 0
  if (x==0) return(0)
  #L_alpha and L_alpha/2
  l1 <- qgamma(alpha,x)/n
    #particular case x = 1
    if (x==1) return(l1)
  l2 <- qgamma(alpha/2,x)/n
  #x^+
  xp <- -qpois(alpha/2,n*l2)+1
  #particular case x^+ = 1
  if (xp==1){
     p <- dpois(0,n*l1)
     if (p>=alpha) return(l1)}
  #\check\1/theta
  if (xp < 1) {oneoverthetamin <- exp((lgamma(x)- lgamma(-xp + 1))/(-xp - x + 1) + log(n))}
         else {oneoverthetamin <- 1/l1}
  u <- upperlimit(0,alpha,epsilon,(1/l1),(1/l2),oneoverthetamin,-x,xp,n)
  return(1/u)}

upperlimit <- function(uplow,alpha,epsilon,u1,u2,thetamin,x,xp,arg){
  #x^- = x^+ ?
  pq <- sumdif(uplow,u1,arg,x,xp-1)
  if (pq[2]>0){
  #solve Delta = 0
     umin=u1
     umax=u2
     pq[2] <- 1
     while (abs(pq[2])>epsilon){
        u= (umin + umax)/2
        pq <- sumdif(uplow,u,arg,x,xp-1)
        if (pq[2]>0){umin <- u} else {umax <- u}}
#print(pq[1])
     pq <- sumdif(uplow,u,arg,x,xp)
     if (pq[1]<alpha) return(u)}

#solve ksi = alpha
  umin=u1
  umax=min(u2,thetamin)
  pq[1] <- 2
  while (abs(pq[1]-alpha)>epsilon){
  u= (umin + umax)/2
  pq <- sumdif(uplow,u,arg,x,xp)
  if (pq[1]>alpha){umin <- u} else {umax <- u}}
  return(u)}

sumdif <- function(uplow,theta,arg,x,y){
  if (uplow == 1) #upper
	{p <- ppois(x,arg*theta)
	 q <- 1-ppois(y-1,arg*theta)}
  else #lower
	{p <- 1-ppois(-x-1,arg/theta)
       q <- ppois(-y,arg/theta)}
  sumdif <- c(p+q,p-q)}