#Compute Blaker's exact confidence interval for the Hypergeometric distribution
#Auteurs/Authors - Bruno LECOUTRE & Jacques POITEVINEAU
#Reference: 
#  Lecoutre B., Poitevineau J. (2012)
#  New results for computing exact confidence intervals for one parameter discrete distributions.
#  hal.archives ouvertes.
#  http://hal.archives-ouvertes.fr/docs/00/91/14/88/PDF/Bruno_Lecoutre_Jacques_Poitevineau_-_New_results_for_computing_exact_confidence_intervals_for_one_parameter_discrete_distributions.pdf 

#bhyperG computes the upper limit of the hypergeometric parameter by successive decrements.

bhyperG <- function(alpha,x,N,n)
{
  epsilon <- .00000000000001
  if (x==n) return(1)
  #U_alpha/2
  u2 <- x+qghyper(1-alpha/2,-x-1,N-n,-n-1)
  #x^+
  xp <- qhyper(1-alpha/2,u2,N-u2,n)+1
#  return(c(u2,xp))
  repeat
   {
     #Pr(X<=x)
     p <- phyper(x,u2,N-u2,n)
     #Pr(X>=xp-1)
     q1 <- 1-phyper(xp-2,u2,N-u2,n)
     #Pr(X>=xp1)
     q <- 1-phyper(xp-1,u2,N-u2,n)
     xi <- p+q
     xi1 <- p+q1
     delta1 <- p-q1
     if ((xi1 >= alpha  && (delta1 >= 0 || x+xp-1==n) ) 
             || (xi >= alpha && delta1 <= 0)) return(u2)
     u2 <- u2-1
   }
}

qghyper <-
function (p, a, k, N, lower.tail = TRUE, log.p = FALSE) 
{
    if (log.p == TRUE) 
        p <- exp(p)
    if (lower.tail == FALSE) 
        p <- 1 - p
    M <- max(length(p), length(a), length(k), length(N))
    p <- rep(p, length.out = M)
    a <- rep(a, length.out = M)
    k <- rep(k, length.out = M)
    N <- rep(N, length.out = M)
    value <- .C("qghyperR", as.double(p), as.double(a), as.double(k), 
        as.double(N), as.integer(M), val = double(M),PACKAGE="SuppDists")$val
    value
}